local tinsert = _G.tinsert;
local ipairs, pairs = _G.ipairs, _G.pairs;
local type = _G.type;
local gmatch, gsub, match = _G.string.gmatch, _G.string.gsub, _G.string.match;

local f = CreateFrame("Frame");

local forbiddenKeys = {
	"_FLOOR_",
	"^A_RANDOM_DUNGEON",
	"ACHIEVEMENT",
	"ADVENTURE",
	"AGONY",
	"AJ_",
	"ALLERIANSTRONGHOLD",
	"ALTAROFSHATAR",
	"ARCHAEOLOGY",
	"ARENA",
	"ARGUS",
	"ARTIFACT",
	"BARBERSHOP",
	"BATTLE_?PET",
	"BLACK_MARKET",
	"BOOST2_",
	"BOUNTY",
	"BRAWL",
	"BUILDING",
	"CALENDAR",
	"CAPACITANCE",
	"CAPTAIN",
	"CHALLENGE",
	"CHAMPION",
	"^CHI",
	"COLLECTION",
	"COMPANION",
	"CORE_ABILITY",
	"CORRUPTION",
	"DAILY",
	"DARK",
	"DEATHKNIGHT",
	"DEMON",
	"_DIFFICULTY_CHANGED?_",
	"DRAEN",
	"^DUNGEON_",
	"_EMBERS",
	"EQUIPMENT_?MANAGER",
	"EQUIPMENT_?SET",
	"EXPANSION",
	"_FEL",
	"FINDER",
	"FLEX_RAID",
	"FOLLOWER",
	"FORGE",
	"FUEL",
	"GARR",
	"_GEM",
	"GEM_",
	"GLYPH",
	"GUILD_EXPERIENCE",
	"GUILDBANK",
	"GUILDEVENT",
	"GUILDINFOTAB",
	"GUILDRECRUITMENT",
	"GUILDS",
	"HAIR",
	"HEIRLOOM",
	"HEROIC",
	"JOURNAL",
	"LEGACY",
	"LEGION",
	"LF[DGMR]",
	"LUNAR",
	"MAELSTROM",
	"MAP_BAR",
	"MONK",
	"PANDAREN",
	"PERK",
	"PET_?BATTLE",
	"PREMADE",
	"PRESTIGE",
	"ORDER_?HALL",
	"RAIDBROWSER",
	"REAGENTBANK",
	"RELIC_",
	"POSTMASTER",
	"RUN[EI]",
	"SCENARIO",
	"SKIRMISH",
	"SOCKET",
	"SPECIALIZATION",
	"TEAM",
	"TOY",
	"TRANSMOG",
	"VEHICLE",
	"WARDROBE",
	"WARGAME",
	"WEEKLY",
	"WINTERGRASP",
	"WORLD_QUEST",
};

local db = {
	['patched'] = {},
	['removed'] = {},
	['unpatched'] = {},
}

local function isForbidden(k, v) 
	if (type(v) ~= 'string') then
		return false;
	end
	
	if (k:upper() ~= k) then
		return false;
	end
	
	for _, name in ipairs(forbiddenKeys) do
		if (k:match(name)) then
			return true;
		end
	end
	
	return false;
end

local function clean()
	for k, v in pairs(_G) do
		if (isForbidden(k, v)) then
			_G[k] = nil;
			table.insert(db.removed, k .. ' --> ' .. v);
		end
	end
	
	table.sort(db.removed);
end

local function fix()
	for k, v in pairs(_G) do
		if (type(v) == 'string' and v:match('%%%d+%$%w')) then
			local replacements = {};
			for kk in gmatch(v, '%%%d+%$%w') do
				tinsert(replacements, kk);
			end

			local replaceTemplate = true;
			
			for kk, vv in ipairs(replacements) do
				if (match(vv, '%d+') ~= tostring(kk)) then
					replaceTemplate = false;
					break;
				end
			end
			
			if (replaceTemplate) then
				while(match(_G[k], '%%%d+%$%w')) do
					_G[k] = gsub(_G[k], '(%%)%d+%$(%w)', '%1%2');
				end
				
				table.insert(db.patched, {k, v, _G[k]});
			else
				table.insert(db.unpatched, k .. ' --> ' .. v);
			end
		end
	end
	
end

local function persist() 
	f:SetScript('OnUpdate', nil);
	
	WoWStringRepairDB = db;
end


f:SetScript('OnUpdate', persist);
clean();
fix();
